@@ -38,6 +38,7 @@ struct _AtspiAccessiblePrivate
   GHashTable *cache;
   guint cache_ref_count;
   guint iteration_stamp;
+  gboolean holds_cache_ref;
 };
 
 GHashTable *
@@ -45,6 +46,9 @@ _atspi_accessible_ref_cache (AtspiAccessible *accessible);
 
 void
 _atspi_accessible_unref_cache (AtspiAccessible *accessible);
+
+void
+_atspi_accessible_set_cached (AtspiAccessible *accessible, gboolean cached);
 G_END_DECLS
 
 #endif /* _ATSPI_ACCESSIBLE_H_ */
