@@ -98,10 +98,7 @@ event_check (GSource *source)
 static void
 xi2keyevent (XIDeviceEvent *xievent, XEvent *xkeyevent)
 {
-                    XkbStateRec st= {};
-
   memset (xkeyevent, 0, sizeof (*xkeyevent));
-                    XkbGetState (xievent->display, XkbUseCoreKbd, &st);
 
   switch (xievent->evtype)
     {
@@ -125,7 +122,7 @@ xi2keyevent (XIDeviceEvent *xievent, XEvent *xkeyevent)
   xkeyevent->xkey.y = xievent->event_y;
   xkeyevent->xkey.x_root = xievent->root_x;
   xkeyevent->xkey.y_root = xievent->root_y;
-  xkeyevent->xkey.state = xievent->mods.effective | (st.group << 13);
+  xkeyevent->xkey.state = xievent->mods.effective;
   xkeyevent->xkey.keycode = xievent->detail;
   xkeyevent->xkey.same_screen = 1;
 }
@@ -346,6 +343,14 @@ set_virtual_modifier (AtspiDeviceX11 *x11_device, gint keycode, gboolean enabled
   refresh_key_grabs (x11_device);
 }
 
+static gboolean
+should_suppress_keygrab_window_events ()
+{
+  /* The goal here is to only enable this hack for i3 and ratpoison, so,
+     as an imperfect proxy, check that XDG_CURRENT_DESKTOP is unset. */
+  return (getenv ("XDG_CURRENT_DESKTOP") == NULL);
+}
+
 static gboolean
 do_event_dispatch (gpointer user_data)
 {
@@ -400,7 +405,8 @@ do_event_dispatch (gpointer user_data)
                     modifiers |= (1 << ATSPI_MODIFIER_NUMLOCK);
                   if (xiDevEv->deviceid == xiDevEv->sourceid)
                     atspi_device_notify_key (ATSPI_DEVICE (device), (xevent.xcookie.evtype == XI_KeyPress), xiRawEv->detail, keysym, modifiers, text);
-                  /* otherwise it's probably a duplicate event from a key grab */
+                  else if (should_suppress_keygrab_window_events ())
+                    _atspi_update_window_filter_time ();
                   XFreeEventData (priv->display, &xevent.xcookie);
                   break;
                 case FocusIn:
@@ -646,8 +652,8 @@ atspi_device_x11_finalize (GObject *object)
       AtspiX11KeyGrab *grab = priv->key_grabs->data;
       disable_key_grab (device, grab);
       g_boxed_free (ATSPI_TYPE_KEY_DEFINITION, grab->kd);
-      priv->key_grabs = g_slist_remove (priv->key_grabs, grab);
       g_free (grab);
+      priv->key_grabs = g_slist_remove (priv->key_grabs, grab);
     }
 
   g_slist_free_full (priv->modifiers, g_free);
@@ -712,11 +718,9 @@ atspi_device_x11_remove_key_grab (AtspiDevice *device, guint id)
         {
           disable_key_grab (x11_device, other);
           priv->key_grabs = g_slist_remove (priv->key_grabs, other);
-          break;
+          return;
         }
     }
-
-  g_free (kd);
 }
 
 static guint
@@ -801,20 +805,17 @@ atspi_device_x11_generate_mouse_event (AtspiDevice *device, AtspiAccessible *obj
   if (p->y == -1 && atspi_accessible_get_role (obj, NULL) == ATSPI_ROLE_APPLICATION)
     {
       g_clear_error (error);
+      g_free (p);
       AtspiAccessible *child = atspi_accessible_get_child_at_index (obj, 0, NULL);
       if (child)
         {
-          g_free (p);
           p = atspi_component_get_position (ATSPI_COMPONENT (child), ATSPI_COORD_TYPE_SCREEN, error);
           g_object_unref (child);
         }
     }
 
   if (p->y == -1 || p->x == -1)
-    {
-      g_free (p);
-      return;
-    }
+    return;
 
   x += p->x;
   y += p->y;
