@@ -34,10 +34,10 @@ G_BEGIN_DECLS
 
 void _atspi_send_event (AtspiEvent *e);
 
-DBusHandlerResult _atspi_dbus_handle_event (DBusConnection *bus, DBusMessage *message);
-void
-_atspi_reregister_event_listeners ();
+void _atspi_dbus_handle_event (DBusMessage *message);
+void _atspi_reregister_event_listeners ();
 
+void _atspi_update_window_filter_time ();
 G_END_DECLS
 
 #endif /* _ATSPI_EVENT_LISTENER_H_ */
